<?php

namespace Drupal\mail_box_management\Plugin;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\node\Entity\Node;

/**
 * EntitySaver.php contains class EntitySaver.
 *
 * @file
 * EntitySaver.php contains class EntitySaver.
 */

/**
 * EntitySaver handles saving of content node for mail_type_content.
 *
 * @class
 * EntitySaver handles saving of content node for mail_type_content.
 */
class EntitySaver {

  /**
   * Entities data to be saved.
   *
   * @var array
   */
  private array $entities = [];

  /**
   * Queue up the data to be saved as nodes.
   *
   * @param array $entities
   *   Validated data of Entities to be saved.
   * @param int $owner_uid
   *   Owner uid.
   *
   * @return void
   *   Nothing is returned.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function add(array $entities, int $owner_uid = 1): void {

    if (array_key_exists(0, $entities)) {
      foreach ($entities as $key => $entity) {
        if (!empty($entity['header']['Subject'])) {

          $attachments = [];
          if (!empty($entity['attachments'])) {
            foreach ($entity['attachments'] as $attachment) {
              $file_type = strtolower($attachment['type']);
              $stored_path = $this->write($file_type, base64_decode($attachment['content']));
              $file = File::create([
                'uri' => $stored_path,
              ]);
              $file->setOwnerId($owner_uid);
              $file->setPermanent();
              $file->save();
              $attachments[] = ['target_id' => $file->id()];
            }
          }

          $this->getArr($entity, $attachments);
        }
      }
    }
    else {
      if (!empty($entities['header']['Subject'])) {
        $entity = $entities;
        $attachments = [];
        if (!empty($entity['attachments'])) {
          foreach ($entity['attachments'] as $key => $attachment) {
            $file_type = strtolower($attachment['type']);
            $stored_path = $this->write($file_type, base64_decode($attachment['content']));
            $file = File::create([
              'uri' => $stored_path,
            ]);
            $file->setOwnerId($owner_uid);
            $file->setPermanent();
            $file->save();
            $attachments[] = ['target_id' => $file->id()];
          }
        }

        $this->getArr($entity, $attachments);
      }
    }
  }

  /**
   * True if nodes saved.
   *
   * @return bool
   *   Returns true if node least was saved.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function save(): bool {

    $flags = [];
    foreach ($this->entities as $entity) {
      $flags[] = !empty(Node::create($entity)->save());
    }
    return in_array(TRUE, $flags);
  }

  /**
   * Write file data for attachments.
   *
   * @param string $file_type
   *   File type of file to be created.
   * @param string $data
   *   Data stream of file.
   *
   * @return string|null
   *   Returned file path. if file was created.
   */
  private function write(string $file_type, string $data): ?string {
    $directory = "public://mailbox_management/attachments";
    $file_system = mail_box_management_service('file_system');
    try {
      if ($file_system->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY)) {
        $storage_path = $directory . '/attachment-' . time() . '.' . $file_type;
        $file_system->saveData($data, $storage_path);
        return $storage_path;
      }
    }
    catch (\Exception) {
    }
    return NULL;
  }

  /**
   * Statically add entity data.
   *
   * @param array $data
   *   Entities data.
   * @param int $owner_id
   *   Uid.
   *
   * @return \Drupal\mail_box_management\Plugin\EntitySaver
   *   Returns its instance.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public static function create(array $data, int $owner_id): EntitySaver {
    $new = new static();
    $new->add($data, $owner_id);
    return $new;
  }

  /**
   * Build up entity array.
   *
   * @param mixed $entity
   *   Entity data.
   * @param array $attachments
   *   Attachments data.
   *
   * @return void
   *   Returns nothing.
   */
  public function getArr(mixed $entity, array $attachments): void {
    $this->entities[] = [
      'title' => $entity['header']['Subject'],
      'subject' => $entity['header']['Subject'],
      'from_email' => $entity['header']['from_email'] ?? '',
      'from_name' => $entity['header']['from_name'] ?? '',
      'to_email' => $entity['header']['to_email'] ?? '',
      'to_name' => $entity['header']['to_name'] ?? '',
      'content' => $entity['body']['content'] ?? '',
      'attachment' => $attachments,
      'type' => 'mail_box_content',
    ];
  }

}
